#include "nodet.h"
#include "globalt.h"

nodet::nodet (void)
{
  //  number of degrees of freedom
  ndofn=0;
  //  number of components in array of gradients
  ncompgrad=0;
  //  number of components in array other
  ncompother = 0;
  //  number of components in array eqother
  ncompeqother=0;
  
  //  type of cross section
  crst = (crsectypet) 0;
  //  cross section id
  idcs=0;
  
  //  number of contributions to the gradients
  ncontr_grad=0;
  //  number of contributions to the fluxes
  ncontr_flux=0;
  //  number of contributions to the array other
  ncontr_other=0;
  //  number of contributions to the array eqother
  ncontr_eqother=0;
  
  //  array of gradients
  gradient=NULL;
  //  array of fluxes
  flux=NULL;
  //  array of auxiliary components
  other=NULL;
  //  array of equilibriated auxiliary components
  eqother=NULL;
  
  
  //  actual nodal values
  nodval=NULL;
  //  nodal values from previous step
  nodvalp=NULL;
  //  initial nodal values
  nodvali=NULL;
  //  time derivatives of nodal values
  nodvalt=NULL;
}

nodet::~nodet (void)
{
  delete [] gradient;
  delete [] flux;
  delete [] other;
  delete [] eqother;
  
  delete [] nodval;
  delete [] nodvalp; 
  delete [] nodvali; 
  delete [] nodvalt;
}

/**
   function reads data stored on nodes
   
   @param in - input file
   @param ndof - number of DOFs on node
   
   JK, 10.7.2008 - revision
*/
void nodet::read (XFILE *in,long ndof)
{
  //  number of DOFs on node
  ndofn=ndof;
  
  //  type of cross section
  xfscanf (in,"%d",(int*)&crst);
  if (crst!=0){
    xfscanf (in,"%ld",&idcs);
    idcs--;
  }
}

/**
   function prints data
   
   @param out - output file
*/
void nodet::print (FILE *out)
{
  //  type of cross section
  fprintf (out," %d ",(int)crst);
  if (crst!=0){
    fprintf (out," %ld\n",idcs+1);
  }
}




/**
   function allocates gradient arrays on nodes
   
   @param ncompo - number components

   18.5.2002
*/
void nodet::alloc_grad (long ncomp)
{
  ncompgrad=ncomp;
  gradient = new double [ndofn*ncomp];
}

/**
   function allocates fluxes arrays on nodes
   
   @param ncompo - number components
   
   18.5.2002
*/
void nodet::alloc_flux (long ncomp)
{
  ncompgrad=ncomp;
  flux = new double [ndofn*ncomp];
}


/**
   function allocates other arrays on nodes
   
   @param ncompo - number components
   
   18.5.2002
*/
void nodet::alloc_other (long ncompo)
{
  ncompother=ncompo;
  other  = new double [ncompo];
}


/**
   function allocates eq_other arrays on nodes
   
   @param ncompo - number components
   
   18.5.2002
*/
void nodet::alloc_eqother (long ncompeqo)
{
  ncompeqother=ncompeqo;
  eqother  = new double [ncompeqo];
}

/**
   function allocates array nodval for actual nodal values on nodes
   
   JK, 10.7.2008
*/
void nodet::alloc_nodval ()
{
  nodval = new double [ndofn];
}

/**
   function allocates array nodvali for initial nodal values on nodes
   
   JK, 10.7.2008
*/
void nodet::alloc_nodvali ()
{
  nodvali = new double [ndofn];
}

/**
   function allocates array nodvalp for previous nodal values on nodes
   
   JK, 10.7.2008
*/
void nodet::alloc_nodvalp ()
{
  nodvalp = new double [ndofn];
}

/**
   function allocates array nodvalt for time derivatives of actual nodal values on nodes
   
   JK, 10.7.2008
*/
void nodet::alloc_nodvalt ()
{
  nodvalt = new double [ndofn];
}





/**
   function sets up all components in array gradient to zero
   
   JK, 10.7.2008
*/
void nodet::nullgrad ()
{
  long i;
  ncontr_grad=0;
  for (i=0;i<ndofn*ncompgrad;i++)
    gradient[i]=0.0;
}

/**
   function sets up all components in array flux to zero
   
   JK, 10.7.2008
*/
void nodet::nullflux ()
{
  long i;
  ncontr_flux=0;
  for (i=0;i<ndofn*ncompgrad;i++)
    flux[i]=0.0;
}

/**
   function sets up all components in array other to zero
   
   @param ntm - number of transported media
*/
void nodet::nullother ()
{
  long i;
  ncontr_other=0;
  for (i=0;i<ncompother;i++)
    other[i]=0.0;
}

/**
   function sets up all components in array eqother to zero
   
   @param ntm - number of transported media
*/
void nodet::nulleqother ()
{
  long i;
  ncontr_eqother=0;
  for (i=0;i<ncompeqother;i++)
    eqother[i]=0.0;
}

/**
   function cleans array nodval
   
   JK, 10.7.2008
*/
void nodet::nullnodval ()
{
  long i;
  for (i=0;i<ndofn;i++){
    nodval[i]=0.0;
  }
}

/**
   function cleans array nodvalp
   
   JK, 10.7.2008
*/
void nodet::nullnodvalp ()
{
  long i;
  for (i=0;i<ndofn;i++){
    nodvalp[i]=0.0;
  }
}

/**
   function cleans array nodvali
   
   JK, 10.7.2008
*/
void nodet::nullnodvali ()
{
  long i;
  for (i=0;i<ndofn;i++){
    nodvali[i]=0.0;
  }
}

/**
   function cleans array nodvalt
   
   JK, 10.7.2008
*/
void nodet::nullnodvalt ()
{
  long i;
  for (i=0;i<ndofn;i++){
    nodvalt[i]=0.0;
  }
}






/**
   function stores gradient components
   
   @param fi - first index
   @param ncomp - number of components
   @param gradv - array containing part of gradient components
   
   19.5.2002
*/
void nodet::storegrad (long fi,long ncomp,vector &gradv)
{
  long i,j;
  
  if (fi==0)  ncontr_grad++;
  
  j=0;
  for (i=fi;i<fi+ncomp;i++){
    gradient[i]+=gradv[j];
    j++;
  }
}

/**
   function stores stress components
   
   @param fi - first index
   @param ncomp- number of components
   @param fluxv - array containing part of flux components
   
   19.5.2002
*/
void nodet::storeflux (long fi,long ncomp,vector &fluxv)
{
  long i,j;
  
  if (fi==0)  ncontr_flux++;
  
  j=0;
  for (i=fi;i<fi+ncomp;i++){
    flux[i]+=fluxv[j];
    j++;
  }
}

/**
   function stores other components

   @param ncomp- number of components
   @param otherv - array containing part of other components
   
   19.5.2002
*/
void nodet::storeother (long ncomp,vector &otherv)
{
  long i;
  
  ncontr_other++;
  
  for (i=0;i<ncomp;i++)
    other[i]+=otherv[i];
}


/**
   function stores eq_other components

   @param ncompeq - number of components
   @param eqotherv - array containing part of eq_other components
   
   19.5.2002
*/
void nodet::storeeqother (long ncompeq,vector &eqotherv)
{
  long i;
  
  ncontr_eqother++;
  
  for (i=0;i<ncompeq;i++)
    eqother[i]+=eqotherv[i];
}



/**
   function computes average gradient at node
*/
void nodet::grad_averageval ()
{
  long i;
  for (i=0;i<ncompgrad;i++){
    gradient[i]/=ncontr_grad;
  }
}

/**
   function computes average flux at node
*/
void nodet::flux_averageval ()
{
  long i;
  for (i=0;i<ncompgrad;i++){
    flux[i]/=ncontr_flux;
  }
}

/**
   function computes average values from array other
*/
void nodet::other_averageval ()
{
  long i;
  for (i=0;i<ncompother;i++){
    other[i]/=ncontr_other;
  }
}

/**
   function computes average values from array eq_other
*/
void nodet::eqother_averageval ()
{
  long i;
  for (i=0;i<ncompeqother;i++){
    eqother[i]/=ncontr_eqother;
  }
}




/**
   function is used in function transmat.cpp::compute_jump

   @param in - array of input data
   
   JK, 7.1.2008
*/
void nodet::give_values (double *in, double *inp, double *ineq)
{
  long i;
  
  for (i=0;i<ndofn;i++){
    in[i]=nodval[i];
  }
  
  for (i=0;i<ndofn;i++){
    inp[i]=nodvalp[i];
  }

  for (i=0;i<ncompeqother-5;i++){
    ineq[i]=eqother[i];
  }
}

/**
   function is used in function transmat.cpp::compute_jump

   @param out - array of saved data
   
   JK, 7.1.2008
*/
void nodet::save_values (double *out)
{
  long i;
  
  for (i=0;i<ncompeqother-5;i++){
    eqother[i]=out[i];
  }
}

/**
   function is used in function transmat.cpp::compute_jump

   function saves nodal values to array nodval
   
   @param nv - array of nodal values
   
   JK, 8.1.2008
*/
void nodet::save_nodval (double *nv)
{
  long i;
  
  for (i=0;i<ndofn;i++){
    nodval[i]=nv[i];
  }
}


/**
   function moves array of actual values to array of previous values
   
   JK, 29.5.2007
*/
void nodet::actual_previous_change ()
{
  long i;
    
  for (i=0;i<ndofn;i++){
    nodvalp[i]=nodval[i];
  }
}

